@@ -6,17 +6,17 @@ ModelExportHelper::ModelExportHelper(QObject *parent) : QObject(parent)
 	resetExportParams();
 }
 
-void ModelExportHelper::resetExportParams(void)
+void ModelExportHelper::resetExportParams()
 {
 	sql_gen_progress=progress=0;
 	db_created=ignore_dup=drop_db=drop_objs=export_canceled=false;
 	simulate=use_tmp_names=db_sql_reenabled=false;
-	created_objs[OBJ_ROLE]=created_objs[OBJ_TABLESPACE]=-1;
+	created_objs[ObjectType::Role]=created_objs[ObjectType::Tablespace]=-1;
 	db_model=nullptr;
 	connection=nullptr;
 	scene=nullptr;
 	zoom=100;
-	show_grid=show_delim=page_by_page=false;
+	show_grid=show_delim=page_by_page=splitted=browsable=false;
 	viewp=nullptr;
 }
 
@@ -26,9 +26,9 @@ void ModelExportHelper::abortExport(Exception &e)
 
 	//When running in a separated thread (other than the main application thread) redirects the error in form of signal
 	if(this->thread() && this->thread()!=qApp->thread())
-		emit s_exportAborted(Exception(e.getErrorMessage(), e.getErrorType(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e));
+		emit s_exportAborted(Exception(e.getErrorMessage(), e.getErrorCode(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e));
 	else
-		throw Exception(e.getErrorMessage(),e.getErrorType(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
+		throw Exception(e.getErrorMessage(),e.getErrorCode(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
 }
 
 void ModelExportHelper::handleSQLError(Exception &e, const QString &sql_cmd, bool ignore_dup)
@@ -39,7 +39,7 @@ void ModelExportHelper::handleSQLError(Exception &e, const QString &sql_cmd, boo
 		emit s_errorIgnored(e.getExtraInfo(), e.getErrorMessage(), sql_cmd);
 	//Raises an excpetion if the error returned by the database is not listed in the ignored list of errors
 	else if(ignored_errors.indexOf(e.getExtraInfo()) < 0)
-		throw Exception(e.getErrorMessage(), e.getErrorType(),__PRETTY_FUNCTION__,__FILE__,__LINE__,&e, sql_cmd);
+		throw Exception(e.getErrorMessage(), e.getErrorCode(),__PRETTY_FUNCTION__,__FILE__,__LINE__,&e, sql_cmd);
 	else
 		errors.push_back(e);
 }
@@ -62,7 +62,7 @@ void ModelExportHelper::setIgnoredErrors(const QStringList &err_codes)
 void ModelExportHelper::exportToSQL(DatabaseModel *db_model, const QString &filename, const QString &pgsql_ver)
 {
 	if(!db_model)
-		throw Exception(ERR_ASG_NOT_ALOC_OBJECT,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+		throw Exception(ErrorCode::AsgNotAllocattedObject,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 
 	connect(db_model, SIGNAL(s_objectLoaded(int,QString,uint)), this, SLOT(updateProgress(int,QString,uint)));
 
@@ -71,18 +71,18 @@ void ModelExportHelper::exportToSQL(DatabaseModel *db_model, const QString &file
 		progress=sql_gen_progress=0;
 		BaseObject::setPgSQLVersion(pgsql_ver);
 		emit s_progressUpdated(progress,
-							   trUtf8("Generating SQL code for PostgreSQL `%1'").arg(BaseObject::getPgSQLVersion()),
-							   BASE_OBJECT);
+							   tr("Generating SQL code for PostgreSQL `%1'").arg(BaseObject::getPgSQLVersion()),
+							   ObjectType::BaseObject);
 		progress=1;
-		db_model->saveModel(filename, SchemaParser::SQL_DEFINITION);
+		db_model->saveModel(filename, SchemaParser::SqlDefinition);
 
-		emit s_progressUpdated(100, trUtf8("Output SQL file `%1' successfully written.").arg(filename), BASE_OBJECT);
+		emit s_progressUpdated(100, tr("Output SQL file `%1' successfully written.").arg(filename), ObjectType::BaseObject);
 		emit s_exportFinished();
 	}
 	catch(Exception &e)
 	{
 		disconnect(db_model, nullptr, this, nullptr);
-		throw Exception(e.getErrorMessage(), e.getErrorType(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
+		throw Exception(e.getErrorMessage(), e.getErrorCode(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
 	}
 
 	disconnect(db_model, nullptr, this, nullptr);
@@ -91,7 +91,7 @@ void ModelExportHelper::exportToSQL(DatabaseModel *db_model, const QString &file
 void ModelExportHelper::exportToPNG(ObjectsScene *scene, const QString &filename, double zoom, bool show_grid, bool show_delim, bool page_by_page, QGraphicsView *viewp)
 {
 	if(!scene)
-		throw Exception(ERR_ASG_NOT_ALOC_OBJECT,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+		throw Exception(ErrorCode::AsgNotAllocattedObject,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 
 	try
 	{
@@ -117,7 +117,9 @@ void ModelExportHelper::exportToPNG(ObjectsScene *scene, const QString &filename
 		scene->clearSelection();
 
 		//Make a backup of the current scene options
-		ObjectsScene::getGridOptions(shw_grd, align_objs, shw_dlm);
+		shw_grd = ObjectsScene::isShowGrid();
+		align_objs = ObjectsScene::isAlignObjectsToGrid();
+		shw_dlm = ObjectsScene::isShowPageDelimiters();
 
 		//Sets the options passed by the user
 		ObjectsScene::setGridOptions(show_grid, false, show_delim);
@@ -146,14 +148,14 @@ void ModelExportHelper::exportToPNG(ObjectsScene *scene, const QString &filename
 			pages=scene->getPagesForPrinting(page_sz, margins.size(), h_cnt, v_cnt);
 
 			//Configures the template filename for pages pixmaps
-			tmpl_filename=fi.absolutePath() + GlobalAttributes::DIR_SEPARATOR + fi.baseName() + QString("_p%1.") + fi.completeSuffix();
+			tmpl_filename=fi.absolutePath() + GlobalAttributes::DirSeparator + fi.baseName() + QString("_p%1.") + fi.completeSuffix();
 		}
 		else
 		{
 			QRectF rect=scene->itemsBoundingRect(true);
 
 			//Give some margin to the resulting image
-			QSizeF margin=QSizeF(5 * BaseObjectView::HORIZ_SPACING, 5 * BaseObjectView::VERT_SPACING);
+			QSizeF margin=QSizeF(5 * BaseObjectView::HorizSpacing, 5 * BaseObjectView::VertSpacing);
 			rect.setTopLeft(rect.topLeft() - QPointF(margin.width(), margin.height()));
 			rect.setSize(rect.size() + margin);
 
@@ -197,8 +199,8 @@ void ModelExportHelper::exportToPNG(ObjectsScene *scene, const QString &filename
 			painter.setRenderHint(QPainter::TextAntialiasing, true);
 			painter.setRenderHint(QPainter::SmoothPixmapTransform, true);
 
-			emit s_progressUpdated((page_idx/static_cast<float>(pages.size())) * 90,
-								   trUtf8("Rendering objects to page %1/%2.").arg(page_idx).arg(pages.size()), BASE_OBJECT);
+			emit s_progressUpdated((page_idx/static_cast<double>(pages.size())) * 90,
+														 tr("Rendering objects to page %1/%2.").arg(page_idx).arg(pages.size()), ObjectType::BaseObject);
 
 			//Render the entire viewport onto the pixmap
 			view->render(&painter, QRectF(QPointF(0,0), pix.size()), retv);
@@ -214,8 +216,8 @@ void ModelExportHelper::exportToPNG(ObjectsScene *scene, const QString &filename
 				ObjectsScene::setGridOptions(shw_grd, align_objs, shw_dlm);
 				scene->update();
 
-				throw Exception(Exception::getErrorMessage(ERR_FILE_DIR_NOT_WRITTEN).arg(file),
-								ERR_FILE_DIR_NOT_WRITTEN,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+				throw Exception(Exception::getErrorMessage(ErrorCode::FileDirectoryNotWritten).arg(file),
+												ErrorCode::FileDirectoryNotWritten,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 			}
 		}
 
@@ -225,25 +227,25 @@ void ModelExportHelper::exportToPNG(ObjectsScene *scene, const QString &filename
 
 		if(!export_canceled)
 		{
-			emit s_progressUpdated(100, trUtf8("Output image `%1' successfully written.").arg(filename), BASE_OBJECT);
+			emit s_progressUpdated(100, tr("Output image `%1' successfully written.").arg(filename), ObjectType::BaseObject);
 			emit s_exportFinished();
 		}
 		else
 			emit s_exportCanceled();
 
 		if(view!=viewp)
-			delete(view);
+			delete view;
 	}
 	catch(Exception &e)
 	{
-		throw Exception(e.getErrorMessage(), e.getErrorType(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
+		throw Exception(e.getErrorMessage(), e.getErrorCode(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
 	}
 }
 
 void ModelExportHelper::exportToSVG(ObjectsScene *scene, const QString &filename, bool show_grid, bool show_delim)
 {
 	if(!scene)
-		throw Exception(ERR_ASG_NOT_ALOC_OBJECT,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+		throw Exception(ErrorCode::AsgNotAllocattedObject,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 
 	bool shw_dlm=false, shw_grd=false, align_objs=false;
 	QSvgGenerator svg_gen;
@@ -251,30 +253,32 @@ void ModelExportHelper::exportToSVG(ObjectsScene *scene, const QString &filename
 	QFileInfo fi(filename);
 
 	//Making a backup of the current scene options
-	ObjectsScene::getGridOptions(shw_grd, align_objs, shw_dlm);	
+	shw_grd = ObjectsScene::isShowGrid();
+	shw_dlm = ObjectsScene::isShowPageDelimiters();
+	align_objs = ObjectsScene::isAlignObjectsToGrid();
 	scene->setBackgroundBrush(Qt::NoBrush);
 
 	//Disabling grid and delimiters
 	ObjectsScene::setGridOptions(show_grid, false, show_delim);
 	scene->update();
 
-	emit s_progressUpdated(0, trUtf8("Exporting model to SVG file."));
+	emit s_progressUpdated(0, tr("Exporting model to SVG file."));
 
 	svg_gen.setFileName(filename);
-	svg_gen.setTitle(trUtf8("SVG representation of database model"));
-	svg_gen.setDescription(trUtf8("SVG file generated by pgModeler"));
+	svg_gen.setTitle(tr("SVG representation of database model"));
+	svg_gen.setDescription(tr("SVG file generated by pgModeler"));
 
 	QPainter *painter=new QPainter(&svg_gen);
 	scene->render(painter, QRectF(0, 0, scene_rect.size().width(), scene_rect.size().height()), scene_rect);
-	delete(painter);
+	delete painter;
 
 	//Restoring the scene settings
 	ObjectsScene::setGridOptions(shw_grd, align_objs, shw_dlm);
 	scene->update();
 
 	if(!fi.exists() || !fi.isWritable() || !fi.isReadable())
-			throw Exception(Exception::getErrorMessage(ERR_FILE_DIR_NOT_WRITTEN).arg(filename),
-											ERR_FILE_DIR_NOT_WRITTEN,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+			throw Exception(Exception::getErrorMessage(ErrorCode::FileDirectoryNotWritten).arg(filename),
+											ErrorCode::FileDirectoryNotWritten,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 
 	QFile svg_file;
 	svg_file.setFileName(filename);
@@ -290,12 +294,12 @@ void ModelExportHelper::exportToSVG(ObjectsScene *scene, const QString &filename
 
 		//Forcing the usage of the font settings defined for BaseObjectView and its subclasses
 		svg_def.replace(font_attr.arg(scene->font().family()),
-										font_attr.arg(BaseObjectView::getFontStyle(ParsersAttributes::GLOBAL).font().family()));
+										font_attr.arg(BaseObjectView::getFontStyle(Attributes::Global).font().family()));
 
 		/* Removing the empty (transparent) backgound object in order to save some space in the file if
 		the grid or delimiter is displayed */
 		if(!show_delim && !show_grid)
-			svg_def.replace(QRegExp("(<image)(.)*(xlink:href)(=)(\")(\\w|=|/|\\+|:|;|,|\n)+(\")( )+(/>)"), QString());
+			svg_def.replace(QRegExp("(<image)(.)*(xlink:href)(=)(\")(\\w|=|/|\\+|:|;|,|\n)+(\")( )+(/>)"), "");
 
 		buf.append(svg_def);
 
@@ -304,7 +308,7 @@ void ModelExportHelper::exportToSVG(ObjectsScene *scene, const QString &filename
 		svg_file.close();
 	}
 
-	emit s_progressUpdated(100, trUtf8("Output file `%1' successfully written.").arg(filename), BASE_OBJECT);
+	emit s_progressUpdated(100, tr("Output file `%1' successfully written.").arg(filename), ObjectType::BaseObject);
 	emit s_exportFinished();
 }
 
@@ -314,81 +318,81 @@ void ModelExportHelper::exportToDBMS(DatabaseModel *db_model, Connection conn, c
 	QString  version, sql_cmd, buf, sql_cmd_comment;
 	Connection new_db_conn;
 	unsigned i, count;
-	ObjectType types[]={OBJ_ROLE, OBJ_TABLESPACE};
+	ObjectType types[]={ObjectType::Role, ObjectType::Tablespace};
 	BaseObject *object=nullptr;
-	QString tmpl_comm_regexp = QString("(COMMENT)( )+(ON)( )+(%1)(.)+(\n)(") + ParsersAttributes::DDL_END_TOKEN + QString(")");
+	QString tmpl_comm_regexp = QString("(COMMENT)( )+(ON)( )+(%1)(.)+(\n)(") + Attributes::DdlEndToken + QString(")");
 	QRegExp comm_regexp;
 
 	try
 	{
 		if(!db_model)
-			throw Exception(ERR_ASG_NOT_ALOC_OBJECT,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+			throw Exception(ErrorCode::AsgNotAllocattedObject,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 
 		/* If the export is called using ignore duplications or drop database and simulation mode at same time
 		an error is raised because the simulate mode (mainly used as SQL validation) cannot
 		undo column addition (this can be changed in the future) */
 		if(simulate && (ignore_dup || drop_db || drop_objs))
-			throw Exception(ERR_MIX_INCOMP_EXPORT_OPTS,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+			throw Exception(ErrorCode::MixingIncompExportOptions,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 		else if(drop_db && drop_objs)
-			throw Exception(ERR_MIX_INCOMP_DROP_OPTS,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+			throw Exception(ErrorCode::MixingIncompDropOptions,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 
 		connect(db_model, SIGNAL(s_objectLoaded(int,QString,uint)), this, SLOT(updateProgress(int,QString,uint)), Qt::DirectConnection);
 
 		export_canceled=false;
 		db_created=false;
 		progress=sql_gen_progress=0;
-		created_objs[OBJ_ROLE]=created_objs[OBJ_TABLESPACE]=-1;
+		created_objs[ObjectType::Role]=created_objs[ObjectType::Tablespace]=-1;
 		errors.clear();
 
 		//Retrive the DBMS version in order to generate the correct code
 		conn.connect();
 		version=conn.getPgSQLVersion(true);
 
-		emit s_progressUpdated(progress, trUtf8("Starting export to DBMS."));
+		emit s_progressUpdated(progress, tr("Starting export to DBMS."));
 
 		//Overriding the DBMS version case the version is informed on parameter
 		if(!pgsql_ver.isEmpty())
 		{
 			BaseObject::setPgSQLVersion(pgsql_ver);
-			emit s_progressUpdated(progress, trUtf8("PostgreSQL version detection overridden. Using version `%1'.").arg(pgsql_ver));
+			emit s_progressUpdated(progress, tr("PostgreSQL version detection overridden. Using version `%1'.").arg(pgsql_ver));
 		}
 		else
 		{
 			BaseObject::setPgSQLVersion(version);
-			emit s_progressUpdated(progress, trUtf8("PostgreSQL `%1' server detected.").arg(version));
+			emit s_progressUpdated(progress, tr("PostgreSQL `%1' server detected.").arg(version));
 		}
 
 		if(simulate && use_tmp_names)
 		{
-			emit s_progressUpdated(progress, trUtf8("Generating temporary names for database, roles and tablespaces."));
+			emit s_progressUpdated(progress, tr("Generating temporary names for database, roles and tablespaces."));
 			generateTempObjectNames(db_model);
 		}
 		else if(use_tmp_names)
-			throw Exception(ERR_INV_USE_TMPNAMES_EXPORT_OPT,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+			throw Exception(ErrorCode::InvUsageTempNamesExportOption,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 
 
 		if(simulate && db_model->isSQLDisabled())
 		{
 			db_model->setSQLDisabled(false);
 			db_sql_reenabled=true;
-			emit s_progressUpdated(progress, trUtf8("Enabling the SQL code for database `%1' to avoid errors.").arg(db_model->getName()));
+			emit s_progressUpdated(progress, tr("Enabling the SQL code for database `%1' to avoid errors.").arg(db_model->getName()));
 		}
 
 
 		if(ignore_dup)
 		{
-			emit s_progressUpdated(progress, trUtf8("Ignoring object duplication errors."));
+			emit s_progressUpdated(progress, tr("Ignoring object duplication errors."));
 
 			//Save the current status for ALTER command generation for table columns/constraints
 			saveGenAtlerCmdsStatus(db_model);
 		}
 
 		if(!ignored_errors.isEmpty())
-			emit s_progressUpdated(progress, trUtf8("Ignoring the following error code(s): `%1'.").arg(ignored_errors.join(", ")));
+			emit s_progressUpdated(progress, tr("Ignoring the following error code(s): `%1'.").arg(ignored_errors.join(", ")));
 
 		if(drop_db)
 		{
-			emit s_progressUpdated(progress, trUtf8("Trying to drop database `%1'.").arg(db_model->getName()));
+			emit s_progressUpdated(progress, tr("Trying to drop database `%1'.").arg(db_model->getName()));
 
 			try
 			{
@@ -404,7 +408,7 @@ void ModelExportHelper::exportToDBMS(DatabaseModel *db_model, Connection conn, c
 		}
 
 		if(simulate)
-			emit s_progressUpdated(progress, trUtf8("Simulation mode activated."));
+			emit s_progressUpdated(progress, tr("Simulation mode activated."));
 
 		//Creates the roles and tablespaces separately from the other objects
 		for(type_id=0; type_id < 2 && !export_canceled; type_id++)
@@ -414,7 +418,7 @@ void ModelExportHelper::exportToDBMS(DatabaseModel *db_model, Connection conn, c
 			for(i=0; i < count && !export_canceled; i++)
 			{
 				object=db_model->getObject(i, types[type_id]);
-				progress=((10 * (type_id+1)) + ((i/static_cast<float>(count)) * 10));
+				progress=((10 * (type_id+1)) + ((i/static_cast<double>(count)) * 10));
 
 				try
 				{
@@ -422,14 +426,14 @@ void ModelExportHelper::exportToDBMS(DatabaseModel *db_model, Connection conn, c
 					{
 						//Emits a signal indicating that the object is being exported
 						emit s_progressUpdated(progress,
-											   trUtf8("Creating object `%1' (%2)")
+											   tr("Creating object `%1' (%2)")
 											   .arg(object->getName())
 											   .arg(object->getTypeName()),
 											   object->getObjectType());
 
-						sql_cmd=object->getCodeDefinition(SchemaParser::SQL_DEFINITION);
+						sql_cmd=object->getCodeDefinition(SchemaParser::SqlDefinition);
 
-						if(types[type_id] == OBJ_TABLESPACE)
+						if(types[type_id] == ObjectType::Tablespace)
 						{
 							comm_regexp = QRegExp(tmpl_comm_regexp.arg(object->getSQLName()));
 							pos = comm_regexp.indexIn(sql_cmd);
@@ -467,11 +471,11 @@ void ModelExportHelper::exportToDBMS(DatabaseModel *db_model, Connection conn, c
 
 				//Creating the database on the DBMS
 				emit s_progressUpdated(progress,
-									   trUtf8("Creating database `%1'")
+									   tr("Creating database `%1'")
 									   .arg(db_model->getName()),
-									   OBJ_DATABASE);
+									   ObjectType::Database);
 
-				sql_cmd=db_model->__getCodeDefinition(SchemaParser::SQL_DEFINITION);
+				sql_cmd=db_model->__getCodeDefinition(SchemaParser::SqlDefinition);
 				pos = comm_regexp.indexIn(sql_cmd);
 
 				/* If we find a comment on statment we should strip it from the DB definition in
@@ -499,19 +503,19 @@ void ModelExportHelper::exportToDBMS(DatabaseModel *db_model, Connection conn, c
 			//Connects to the new created database to create the other objects
 			progress=20;
 			new_db_conn=conn;
-			new_db_conn.setConnectionParam(Connection::PARAM_DB_NAME, db_model->getName());
+			new_db_conn.setConnectionParam(Connection::ParamDbName, db_model->getName());
 			emit s_progressUpdated(progress,
-								   trUtf8("Connecting to database `%1'")
+								   tr("Connecting to database `%1'")
 								   .arg(db_model->getName()));
 
 			new_db_conn.connect();
 			progress=30;
 
 			//Creating the other object types
-			emit s_progressUpdated(progress, trUtf8("Generating SQL for `%1' objects...").arg(db_model->getObjectCount()));
+			emit s_progressUpdated(progress, tr("Generating SQL for `%1' objects...").arg(db_model->getObjectCount()));
 
 			//Exporting the database model definition using the opened connection
-			buf=db_model->getCodeDefinition(SchemaParser::SQL_DEFINITION, false);
+			buf=db_model->getCodeDefinition(SchemaParser::SqlDefinition, false);
 			progress=40;
 			exportBufferToDBMS(buf, new_db_conn, drop_objs);
 		}
@@ -560,13 +564,13 @@ void ModelExportHelper::exportToDBMS(DatabaseModel *db_model, Connection conn, c
 		if(this->thread() && this->thread()!=qApp->thread())
 		{
 			errors.push_back(e);
-			emit s_exportAborted(Exception(e.getErrorMessage(), e.getErrorType(),__PRETTY_FUNCTION__,__FILE__,__LINE__, errors));
+			emit s_exportAborted(Exception(e.getErrorMessage(), e.getErrorCode(),__PRETTY_FUNCTION__,__FILE__,__LINE__, errors));
 		}
 		else
 		{
 			//Redirects any error to terrorsr
 			if(errors.empty())
-				throw Exception(e.getErrorMessage(),e.getErrorType(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
+				throw Exception(e.getErrorMessage(),e.getErrorCode(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
 			else
 			{
 				errors.push_back(e);
@@ -576,18 +580,49 @@ void ModelExportHelper::exportToDBMS(DatabaseModel *db_model, Connection conn, c
 	}
 }
 
+void ModelExportHelper::exportToDataDict(DatabaseModel *db_model, const QString &path, bool browsable, bool splitted)
+{
+	if(!db_model)
+		throw Exception(ErrorCode::AsgNotAllocattedObject,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+
+	connect(db_model, SIGNAL(s_objectLoaded(int,QString,uint)), this, SLOT(updateProgress(int,QString,uint)));
+
+	try
+	{
+		progress=0;
+		emit s_progressUpdated(progress,
+													 tr("Starting data dictionary generation..."),
+													 ObjectType::BaseObject);
+		progress=1;
+		db_model->saveDataDictionary(path, browsable, splitted);
+
+		emit s_progressUpdated(100, tr("Data dictionary successfully saved into `%1'.").arg(path), ObjectType::BaseObject);
+		emit s_exportFinished();
+	}
+	catch(Exception &e)
+	{
+		disconnect(db_model, nullptr, this, nullptr);
+		throw Exception(e.getErrorMessage(), e.getErrorCode(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
+	}
+
+	disconnect(db_model, nullptr, this, nullptr);
+}
+
 void ModelExportHelper::saveGenAtlerCmdsStatus(DatabaseModel *db_model)
 {
 	vector<BaseObject *> objects;
-	Table *tab=nullptr;
+	PhysicalTable *tab=nullptr;
 	Relationship *rel=nullptr;
 
-	objects.insert(objects.end(), db_model->getObjectList(OBJ_TABLE)->begin(),
-				   db_model->getObjectList(OBJ_TABLE)->end());
+	objects.insert(objects.end(), db_model->getObjectList(ObjectType::Table)->begin(),
+								 db_model->getObjectList(ObjectType::Table)->end());
+
+	objects.insert(objects.end(), db_model->getObjectList(ObjectType::ForeignTable)->begin(),
+								 db_model->getObjectList(ObjectType::ForeignTable)->end());
 
 	//Store the relationship on the auxiliary vector but only many-to-many are considered
-	objects.insert(objects.end(), db_model->getObjectList(OBJ_RELATIONSHIP)->begin(),
-				   db_model->getObjectList(OBJ_RELATIONSHIP)->end());
+	objects.insert(objects.end(), db_model->getObjectList(ObjectType::Relationship)->begin(),
+				   db_model->getObjectList(ObjectType::Relationship)->end());
 
 	alter_cmds_status.clear();
 
@@ -600,7 +635,7 @@ void ModelExportHelper::saveGenAtlerCmdsStatus(DatabaseModel *db_model)
 		if(rel)
 			tab=rel->getGeneratedTable();
 		else
-			tab=dynamic_cast<Table *>(objects.back());
+			tab=dynamic_cast<PhysicalTable *>(objects.back());
 
 		if(tab)
 		{
@@ -614,15 +649,10 @@ void ModelExportHelper::saveGenAtlerCmdsStatus(DatabaseModel *db_model)
 	}
 }
 
-void ModelExportHelper::restoreGenAtlerCmdsStatus(void)
+void ModelExportHelper::restoreGenAtlerCmdsStatus()
 {
-	map<Table *, bool>::iterator itr=alter_cmds_status.begin();
-
-	while(itr!=alter_cmds_status.end())
-	{
-		itr->first->setGenerateAlterCmds(itr->second);
-		itr++;
-	}
+	for(auto &itr : alter_cmds_status)
+		itr.first->setGenerateAlterCmds(itr.second);
 
 	alter_cmds_status.clear();
 }
@@ -630,14 +660,14 @@ void ModelExportHelper::restoreGenAtlerCmdsStatus(void)
 void ModelExportHelper::undoDBMSExport(DatabaseModel *db_model, Connection &conn, bool use_tmp_names)
 {
 	QString drop_cmd=QString("DROP %1 %2;");
-	ObjectType types[]={OBJ_ROLE, OBJ_TABLESPACE};
+	ObjectType types[]={ObjectType::Role, ObjectType::Tablespace};
 	int type_id;
 	BaseObject *object=nullptr;
 
 	//In case of error during the export all created object are removed
-	if(db_created || created_objs[OBJ_ROLE] >= 0 || created_objs[OBJ_TABLESPACE] >= 0)
+	if(db_created || created_objs[ObjectType::Role] >= 0 || created_objs[ObjectType::Tablespace] >= 0)
 	{
-		emit s_progressUpdated(99, trUtf8("Destroying objects created on the server."));
+		emit s_progressUpdated(99, tr("Destroying objects created on the server."));
 
 		//Dropping the database
 		if(db_created)
@@ -663,7 +693,7 @@ void ModelExportHelper::undoDBMSExport(DatabaseModel *db_model, Connection &conn
 
 		if(use_tmp_names)
 		{
-			emit s_progressUpdated(100, trUtf8("Restoring original names of database, roles and tablespaces."));
+			emit s_progressUpdated(100, tr("Restoring original names of database, roles and tablespaces."));
 			restoreObjectNames();
 		}
 	}
@@ -681,20 +711,20 @@ void ModelExportHelper::generateTempObjectNames(DatabaseModel *db_model)
 	QTextStream stream(&tmp_name);
 	QDateTime dt=QDateTime::currentDateTime();
 	QCryptographicHash hash(QCryptographicHash::Md5);
-	map<ObjectType, QString> obj_suffixes={ { OBJ_DATABASE, QString("db_") },
-											{ OBJ_ROLE, QString("rl_")},
-											{ OBJ_TABLESPACE, QString("tb_")} };
+	map<ObjectType, QString> obj_suffixes={ { ObjectType::Database, QString("db_") },
+											{ ObjectType::Role, QString("rl_")},
+											{ ObjectType::Tablespace, QString("tb_")} };
 
 	orig_obj_names.clear();
 	orig_obj_names[db_model]=db_model->getName();
 
-	for(auto &role : *db_model->getObjectList(OBJ_ROLE))
+	for(auto &role : *db_model->getObjectList(ObjectType::Role))
 	{
 		if(!role->isSystemObject())
 			orig_obj_names[role]=role->getName();
 	}
 
-	for(auto &tabspc : *db_model->getObjectList(OBJ_TABLESPACE))
+	for(auto &tabspc : *db_model->getObjectList(ObjectType::Tablespace))
 	{
 		if(!tabspc->isSystemObject())
 			orig_obj_names[tabspc]=tabspc->getName();
@@ -713,7 +743,7 @@ void ModelExportHelper::generateTempObjectNames(DatabaseModel *db_model)
 		obj.first->setName(tmp_name.mid(0,15));
 		tmp_name.clear();
 
-		emit s_progressUpdated(progress, trUtf8("Renaming `%1' (%2) to `%3'")
+		emit s_progressUpdated(progress, tr("Renaming `%1' (%2) to `%3'")
 							   .arg(old_name)
 							   .arg(obj.first->getTypeName())
 							   .arg(obj.first->getName()));
@@ -724,14 +754,15 @@ void ModelExportHelper::generateTempObjectNames(DatabaseModel *db_model)
 	db_model->setCodesInvalidated();
 }
 
-void ModelExportHelper::restoreObjectNames(void)
+void ModelExportHelper::restoreObjectNames()
 {
 	for(auto &obj : orig_obj_names)
 		obj.first->setName(obj.second);
 
 	/* Invalidates the codes of all objects on database model in order to generate the SQL referencing the
 		 object's with their original names */
-	db_model->setCodesInvalidated();
+	if(db_model)
+		db_model->setCodesInvalidated();
 }
 
 bool ModelExportHelper::isDuplicationError(const QString &error_code)
@@ -751,19 +782,18 @@ bool ModelExportHelper::isDuplicationError(const QString &error_code)
 									QString("42P07"), QString("42710"), QString("42701"),
 									QString("42P16")};
 
-	return(err_codes.contains(error_code));
+	return err_codes.contains(error_code);
 }
 
 void ModelExportHelper::exportBufferToDBMS(const QString &buffer, Connection &conn, bool drop_objs)
 {
 	Connection aux_conn;
 	QString sql_buf=buffer, sql_cmd, aux_cmd, lin, msg,
-			obj_name, obj_tp_name, tab_name,
+			obj_name, obj_tp_name, tab_name, orig_conn_db_name,
 			alter_tab=QString("ALTER TABLE");
-	//vector<Exception> errors;
 	vector<QString> db_sql_cmds;
 	QTextStream ts;
-	ObjectType obj_type=BASE_OBJECT;
+	ObjectType obj_type=ObjectType::BaseObject;
 	bool ddl_tk_found=false, is_create=false, is_drop=false;
 	unsigned aux_prog=0, curr_size=0, buf_size=sql_buf.size(),
 			factor=(db_name.isEmpty() ? 70 : 90);
@@ -776,14 +806,14 @@ void ModelExportHelper::exportBufferToDBMS(const QString &buffer, Connection &co
 			drop_tab_obj_reg(QString("^((\\-\\-)+( )*)+(%1)(.)+(DROP)(.)+").arg(alter_tab)),
 			reg_aux;
 
-	vector<ObjectType> obj_types={ OBJ_ROLE, OBJ_FUNCTION, OBJ_TRIGGER, OBJ_INDEX,
-								   OBJ_RULE,	OBJ_TABLE, OBJ_VIEW, OBJ_DOMAIN,
-								   OBJ_SCHEMA,	OBJ_AGGREGATE, OBJ_OPFAMILY,
-								   OBJ_OPCLASS, OBJ_OPERATOR,  OBJ_SEQUENCE,
-								   OBJ_CONVERSION, OBJ_CAST,	OBJ_LANGUAGE,
-								   OBJ_COLLATION, OBJ_EXTENSION, OBJ_TYPE,
-								   OBJ_EVENT_TRIGGER, OBJ_DATABASE };
-
+	vector<ObjectType> obj_types={ ObjectType::Role, ObjectType::Function, ObjectType::Trigger, ObjectType::Index,
+																 ObjectType::Policy, ObjectType::Rule,	ObjectType::Table, ObjectType::View, ObjectType::Domain,
+																 ObjectType::Schema,	ObjectType::Aggregate, ObjectType::OpFamily,
+																 ObjectType::OpClass, ObjectType::Operator,  ObjectType::Sequence,
+																 ObjectType::Conversion, ObjectType::Cast,	ObjectType::Language,
+																 ObjectType::Collation, ObjectType::Extension, ObjectType::Type,
+																 ObjectType::EventTrigger, ObjectType::ForeignDataWrapper, ObjectType::ForeignServer,
+																 ObjectType::UserMapping, ObjectType::ForeignTable, ObjectType::Database, ObjectType::BaseObject };
 
 	/* Extract each SQL command from the buffer and execute them separately. This is done
    to permit the user, in case of error, identify what object is wrongly configured. */
@@ -791,8 +821,10 @@ void ModelExportHelper::exportBufferToDBMS(const QString &buffer, Connection &co
 
 	if(!conn.isStablished())
 	{
+		orig_conn_db_name = conn.getConnectionParam(Connection::ParamDbName);
+
 		if(!db_name.isEmpty())
-			conn.setConnectionParam(Connection::PARAM_DB_NAME, db_name);
+			conn.setConnectionParam(Connection::ParamDbName, db_name);
 
 		conn.connect();
 	}
@@ -804,7 +836,7 @@ void ModelExportHelper::exportBufferToDBMS(const QString &buffer, Connection &co
 			//Cleanup single line comments
 			lin=ts.readLine();
 			curr_size+=lin.size();
-			aux_prog=progress + ((curr_size/static_cast<float>(buf_size)) * factor);
+			aux_prog=progress + ((curr_size/static_cast<double>(buf_size)) * factor);
 
 			/* If the simulation mode is off and the drop objects option is checked,
 		 check if the current line matches one of the accepted drop commands
@@ -824,7 +856,7 @@ void ModelExportHelper::exportBufferToDBMS(const QString &buffer, Connection &co
 			}
 			else
 			{
-				ddl_tk_found=(lin.indexOf(ParsersAttributes::DDL_END_TOKEN) >= 0);
+				ddl_tk_found=(lin.indexOf(Attributes::DdlEndToken) >= 0);
 				lin.remove(QRegExp(QString("^(--)+(.)+$")));
 
 				//If the line isn't empty after cleanup it will be included on sql command
@@ -843,7 +875,7 @@ void ModelExportHelper::exportBufferToDBMS(const QString &buffer, Connection &co
 				{
 					aux_cmd.remove('"');
 					aux_cmd.remove(QString("IF EXISTS "));
-					obj_type=(aux_cmd.contains(QString("COLUMN")) ? OBJ_COLUMN : OBJ_CONSTRAINT);
+					obj_type=(aux_cmd.contains(QString("COLUMN")) ? ObjectType::Column : ObjectType::Constraint);
 					reg_aux=QRegExp(QString("(COLUMN|CONSTRAINT)( )+"));
 
 					//Extracting the table name
@@ -869,9 +901,9 @@ void ModelExportHelper::exportBufferToDBMS(const QString &buffer, Connection &co
 					obj_name=tab_name + QString(".") + obj_name;
 
 					if(is_drop)
-						msg=trUtf8("Dropping object `%1' (%2)").arg(obj_name).arg(BaseObject::getTypeName(obj_type));
+						msg=tr("Dropping object `%1' (%2)").arg(obj_name).arg(BaseObject::getTypeName(obj_type));
 					else
-						msg=trUtf8("Creating object `%1' (%2)").arg(obj_name).arg(BaseObject::getTypeName(obj_type));
+						msg=tr("Creating object `%1' (%2)").arg(obj_name).arg(BaseObject::getTypeName(obj_type));
 
 					emit s_progressUpdated(aux_prog, msg, obj_type, sql_cmd);
 					is_drop=false;
@@ -888,24 +920,23 @@ void ModelExportHelper::exportBufferToDBMS(const QString &buffer, Connection &co
 
 						obj_type=obj_tp;
 
-						//Appeding special tokens when the object is an index or view
 						if(lin.startsWith(QString("CREATE")) || lin.startsWith(QString("ALTER")))
 						{
-							if(obj_tp==OBJ_INDEX)
+							if(obj_tp==ObjectType::Index)
 							{
 								lin.remove(QString("UNIQUE"));
 								lin.remove(QString("CONCURRENTLY"));
 							}
-							else if(obj_tp==OBJ_VIEW)
+							else if(obj_tp==ObjectType::View)
 							{
 								lin.remove(QString("MATERIALIZED"));
 								lin.remove(QString("RECURSIVE"));
 							}
-							else if(obj_tp==OBJ_TABLE)
+							else if(obj_tp==ObjectType::Table)
 							{
 								lin.remove(QString("UNLOGGED"));
 							}
-							else if(obj_tp==OBJ_FUNCTION)
+							else if(obj_tp==ObjectType::Function)
 							{
 								lin.remove(QString("OR REPLACE"));
 							}
@@ -931,33 +962,54 @@ void ModelExportHelper::exportBufferToDBMS(const QString &buffer, Connection &co
 							lin=lin.mid(reg_aux.matchedLength(), sql_cmd.indexOf('\n')).simplified();
 							lin.remove('"');
 
-							if(obj_tp!=OBJ_CAST)
+							if(obj_tp != ObjectType::BaseObject)
 							{
-								int spc_idx=lin.indexOf(' ');
-								obj_name=lin.mid(0, (spc_idx >= 0 ? spc_idx + 1 : lin.size()));
-
-								if(obj_tp!=OBJ_FUNCTION)
+								if(obj_tp!=ObjectType::Cast && obj_tp != ObjectType::UserMapping)
 								{
-									obj_name=obj_name.remove('(').simplified();
-									obj_name=obj_name.remove(')').simplified();
+									int spc_idx=lin.indexOf(' ');
+									obj_name=lin.mid(0, (spc_idx >= 0 ? spc_idx + 1 : lin.size()));
+
+									if(obj_tp!=ObjectType::Function)
+									{
+										obj_name=obj_name.remove('(').simplified();
+										obj_name=obj_name.remove(')').simplified();
+									}
 								}
+								else if(obj_tp == ObjectType::UserMapping)
+								{
+									obj_name.prepend(lin.remove(QString("FOR")).trimmed() + QChar('@'));
+								}
+								else
+								{
+									obj_name=QString("cast") + lin.replace(QString(" AS "),QString(","));
+								}
+
+								//Stores the object type name
+								obj_tp_name=BaseObject::getTypeName(obj_tp);
+								obj_name.remove(';');
+
+								if(is_create)
+									msg=tr("Creating object `%1' (%2)").arg(obj_name).arg(obj_tp_name);
+								else if(is_drop)
+									msg=tr("Dropping object `%1' (%2)").arg(obj_name).arg(obj_tp_name);
+								else
+									msg=tr("Changing object `%1' (%2)").arg(obj_name).arg(obj_tp_name);
 							}
+							// If the type of the object being create can't be identified
 							else
 							{
-								obj_name=QString("cast") + lin.replace(QString(" AS "),QString(","));
+								QString aux_cmd_type;
+
+								if(is_create)
+									aux_cmd_type = QString("CREATE");
+								else if(is_drop)
+									aux_cmd_type = QString("DROP");
+								else
+									aux_cmd_type = QString("ALTER");
+
+								msg=tr("Running auxiliary `%1' command...").arg(aux_cmd_type);
 							}
 
-							//Stores the object type name
-							obj_tp_name=BaseObject::getTypeName(obj_tp);
-							obj_name.remove(';');
-
-							if(is_create)
-								msg=trUtf8("Creating object `%1' (%2)").arg(obj_name).arg(obj_tp_name);
-							else if(is_drop)
-								msg=trUtf8("Dropping object `%1' (%2)").arg(obj_name).arg(obj_tp_name);
-							else
-								msg=trUtf8("Changing object `%1' (%2)").arg(obj_name).arg(obj_tp_name);
-
 							break;
 						}
 					}
@@ -969,13 +1021,13 @@ void ModelExportHelper::exportBufferToDBMS(const QString &buffer, Connection &co
 				else if(!sql_cmd.trimmed().isEmpty())
 				{
 					//General commands like grant, revoke or set aren't explicitly shown
-					emit s_progressUpdated(aux_prog, trUtf8("Running auxiliary command."), BASE_OBJECT, sql_cmd);
+					emit s_progressUpdated(aux_prog, tr("Running auxiliary command."), ObjectType::BaseObject, sql_cmd);
 				}
 
 				//Executes the extracted SQL command
 				if(!sql_cmd.isEmpty())
 				{
-					if(obj_type!=OBJ_DATABASE)
+					if(obj_type != ObjectType::Database)
 						conn.executeDDLCommand(sql_cmd);
 					else
 						db_sql_cmds.push_back(sql_cmd);
@@ -989,6 +1041,10 @@ void ModelExportHelper::exportBufferToDBMS(const QString &buffer, Connection &co
 			{
 				conn.close();
 				aux_conn=conn;
+
+				if(!orig_conn_db_name.isEmpty())
+					aux_conn.setConnectionParam(Connection::ParamDbName, orig_conn_db_name);
+
 				aux_conn.connect();
 				for(QString cmd : db_sql_cmds)
 					aux_conn.executeDDLCommand(cmd);
@@ -997,9 +1053,7 @@ void ModelExportHelper::exportBufferToDBMS(const QString &buffer, Connection &co
 		catch(Exception &e)
 		{
 			if(ddl_tk_found) ddl_tk_found=false;
-
 			handleSQLError(e, sql_cmd, ignore_dup);
-
 			sql_cmd.clear();
 		}
 	}
@@ -1069,7 +1123,15 @@ void ModelExportHelper::setExportToSVGParams(ObjectsScene *scene, const QString
 	this->show_delim=show_delim;
 }
 
-void ModelExportHelper::exportToDBMS(void)
+void ModelExportHelper::setExportToDataDictParams(DatabaseModel *db_model, const QString &path, bool browsable, bool splitted)
+{
+	this->db_model=db_model;
+	this->filename=path;
+	this->browsable=browsable;
+	this->splitted=splitted;
+}
+
+void ModelExportHelper::exportToDBMS()
 {
 	if(connection)
 	{
@@ -1090,7 +1152,7 @@ void ModelExportHelper::exportToDBMS(void)
 	}
 }
 
-void ModelExportHelper::exportToPNG(void)
+void ModelExportHelper::exportToPNG()
 {
 	try
 	{
@@ -1103,7 +1165,7 @@ void ModelExportHelper::exportToPNG(void)
 	}
 }
 
-void ModelExportHelper::exportToSVG(void)
+void ModelExportHelper::exportToSVG()
 {
 	try
 	{
@@ -1116,7 +1178,7 @@ void ModelExportHelper::exportToSVG(void)
 	}
 }
 
-void ModelExportHelper::exportToSQL(void)
+void ModelExportHelper::exportToSQL()
 {
 	try
 	{
@@ -1129,7 +1191,20 @@ void ModelExportHelper::exportToSQL(void)
 	}
 }
 
-void ModelExportHelper::cancelExport(void)
+void ModelExportHelper::exportToDataDict()
+{
+	try
+	{
+		exportToDataDict(db_model, filename, browsable, splitted);
+		resetExportParams();
+	}
+	catch(Exception &e)
+	{
+		abortExport(e);
+	}
+}
+
+void ModelExportHelper::cancelExport()
 {
 	export_canceled=true;
 }
