@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2018 - Raphael Araújo e Silva <raphael@pgmodeler.com.br>
+# Copyright 2006-2020 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,13 +18,14 @@
 
 #include "syntaxhighlighter.h"
 #include "numberedtexteditor.h"
+#include "qtcompat/qplaintexteditcompat.h"
 
 QFont SyntaxHighlighter::default_font=QFont(QString("Source Code Pro"), 10);
 
 SyntaxHighlighter::SyntaxHighlighter(QPlainTextEdit *parent, bool single_line_mode, bool use_custom_tab_width) : QSyntaxHighlighter(parent)
 {
 	if(!parent)
-		throw Exception(ERR_ASG_NOT_ALOC_OBJECT,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+		throw Exception(ErrorCode::AsgNotAllocattedObject,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 
 	this->setDocument(parent->document());
 	this->single_line_mode=single_line_mode;
@@ -32,13 +33,13 @@ SyntaxHighlighter::SyntaxHighlighter(QPlainTextEdit *parent, bool single_line_mo
 	parent->installEventFilter(this);
 
 	if(use_custom_tab_width)
-		parent->setTabStopWidth(NumberedTextEditor::getTabWidth());
+		QtCompat::setTabStopDistance(parent, NumberedTextEditor::getTabDistance());
 
 	//Adjusting the size of the parent input according to the current font size
 	if(single_line_mode)
 	{
 		QFontMetrics fm=QFontMetrics(default_font);
-		int height=fm.height() + (fm.lineSpacing()/static_cast<float>(1.3));
+		int height=fm.height() + (fm.lineSpacing()/static_cast<double>(1.3));
 		parent->setMinimumHeight(height);
 		parent->setMaximumHeight(height);
 		parent->setSizePolicy(parent->sizePolicy().horizontalPolicy(), QSizePolicy::Fixed);
@@ -55,7 +56,7 @@ bool SyntaxHighlighter::eventFilter(QObject *object, QEvent *event)
 			 dynamic_cast<QKeyEvent *>(event)->key()==Qt::Key_Enter))
 	{
 		event->ignore();
-		return(true);
+		return true;
 	}
 
 	/* If the user is about press Control to paste contents or Right mouse button in
@@ -72,15 +73,15 @@ bool SyntaxHighlighter::eventFilter(QObject *object, QEvent *event)
 			qApp->clipboard()->setText(qApp->clipboard()->mimeData()->text());
 	}
 
-	return(QSyntaxHighlighter::eventFilter(object, event));
+	return QSyntaxHighlighter::eventFilter(object, event);
 }
 
 bool SyntaxHighlighter::hasInitialAndFinalExprs(const QString &group)
 {
-	return(initial_exprs.count(group) && final_exprs.count(group));
+	return (initial_exprs.count(group) && final_exprs.count(group));
 }
 
-void SyntaxHighlighter::configureAttributes(void)
+void SyntaxHighlighter::configureAttributes()
 {
 	conf_loaded=false;
 }
@@ -100,26 +101,29 @@ void SyntaxHighlighter::highlightBlock(const QString &txt)
 		//Reset the block's info to permit the rehighlighting
 		info=dynamic_cast<BlockInfo *>(currentBlockUserData());
 		info->resetBlockInfo();
-		setCurrentBlockState(SIMPLE_BLOCK);
+		setCurrentBlockState(SimpleBlock);
 	}
 
 	/* If the previous block info is a open multiline expression the current block will inherit this settings
 	 to force the same text formatting */
-	if(prev_info && currentBlock().previous().userState()==OPEN_EXPR_BLOCK)
+	if(prev_info && currentBlock().previous().userState()==OpenExprBlock &&
+		 currentBlockState() == OpenExprBlock)
 	{
 		info->group=prev_info->group;
 		info->has_exprs=prev_info->has_exprs;
 		info->is_expr_closed=false;
-		setCurrentBlockState(OPEN_EXPR_BLOCK);
+		setCurrentBlockState(OpenExprBlock);
 	}
 
 	if(!txt.isEmpty())
 	{
-		QString text=txt + QChar('\n'), word, group;
+		QString text, word, group;
 		unsigned i=0, len, idx=0, i1;
 		int match_idx, match_len, aux_len, start_col;
 		QChar chr_delim, lookahead_chr;
-		len=text.length();
+
+		text = txt + QString("\n");
+		len = text.length();
 
 		do
 		{
@@ -135,31 +139,71 @@ void SyntaxHighlighter::highlightBlock(const QString &txt)
 				if(word_separators.contains(text[i]))
 				{
 					while(i < len && word_separators.contains(text[i]))
-						word+=text[i++];
+					{
+						word += text[i];
+						i++;
+					}
 				}
 				//If the char is a word delimiter
 				else if(word_delimiters.contains(text[i]))
 				{
-					chr_delim=text[i++];
-					word+=chr_delim;
+					chr_delim = text[i++];
+					word += chr_delim;
 
-					while(i < len && chr_delim!=text[i])
-						word+=text[i++];
-
-					if(i < len && text[i]==chr_delim)
+					while(i < len && chr_delim != text[i])
 					{
-						word+=chr_delim;
+						word += text[i];
+						i++;
+					}
+
+					if(i < len && text[i] == chr_delim)
+					{
+						word += chr_delim;
 						i++;
 					}
 				}
 				else
 				{
+					BlockInfo *prev_info = dynamic_cast<BlockInfo *>(currentBlock().previous().userData());
+
 					while(i < len &&
 						  !word_separators.contains(text[i]) &&
-						  !word_delimiters.contains(text[i]) &&
-						  !ignored_chars.contains(text[i]))
+							!ignored_chars.contains(text[i]) &&
+							!word_delimiters.contains(text[i]))
 					{
-						word+=text[i++];
+						word += text[i];
+						i++;
+					}
+
+					/* This is an workaround for multi lined groups which use word delimiters
+					in their final expressions. In some cases the highlighter can't undertand that
+					a multi line group was closed and right after another group starts, this way it
+					continues to highlight text as the previous multi lined group.
+
+					An example of that situation is for multi lined string group:
+
+					word delimiter: ' (apostrophe)
+					initial-exp: (')(.)*(\n)
+					final-exp: (.)*(')(\n)*
+
+					String:	'lorem\n ipsum' nextword
+
+					In the example above, without the workaround, the highlighter would highlight the first line
+					"'lorem\n" as string and continue to hightlight the " ipsum' nextword" in the same way as well,
+					this because the final expression of the group contains the word delimiter '. In order to force the highlight stop
+					in the last ' we include it in the current evaluated word and increment the position in the text so the next
+					word starts without the word delimiter. */
+					if(i < len && word_delimiters.contains(text[i]) && prev_info && !prev_info->group.isEmpty() && prev_info->has_exprs)
+					{
+						for(auto exp : final_exprs[prev_info->group])
+						{
+							if(exp.pattern().contains(text[i]))
+							{
+								word += text[i];
+								i++;
+								break;
+							}
+						}
 					}
 				}
 			}
@@ -177,6 +221,7 @@ void SyntaxHighlighter::highlightBlock(const QString &txt)
 
 				match_idx=-1;
 				match_len=0;
+
 				group=identifyWordGroup(word, lookahead_chr, match_idx, match_len);
 
 				if(!group.isEmpty())
@@ -186,9 +231,9 @@ void SyntaxHighlighter::highlightBlock(const QString &txt)
 				}
 
 				if(info->has_exprs && !info->is_expr_closed && hasInitialAndFinalExprs(group))
-					setCurrentBlockState(OPEN_EXPR_BLOCK);
+					setCurrentBlockState(OpenExprBlock);
 				else
-					setCurrentBlockState(SIMPLE_BLOCK);
+					setCurrentBlockState(SimpleBlock);
 
 				aux_len=(match_idx + match_len);
 				if(match_idx >=0 &&  aux_len != word.length())
@@ -230,7 +275,7 @@ QString SyntaxHighlighter::identifyWordGroup(const QString &word, const QChar &l
 		info->has_exprs=hasInitialAndFinalExprs(group);
 		info->group=group;
 
-		return(group);
+		return group;
 	}
 	else
 	{
@@ -245,7 +290,7 @@ QString SyntaxHighlighter::identifyWordGroup(const QString &word, const QChar &l
 		}
 
 		if(!match)
-			return(QString());
+			return "";
 		else
 		{
 			info->group=group;
@@ -254,7 +299,7 @@ QString SyntaxHighlighter::identifyWordGroup(const QString &word, const QChar &l
 				info->has_exprs=hasInitialAndFinalExprs(group);
 
 			info->is_expr_closed=false;
-			return(group);
+			return group;
 		}
 	}
 }
@@ -297,15 +342,15 @@ bool SyntaxHighlighter::isWordMatchGroup(const QString &word, const QString &gro
 		if(match) break;
 	}
 
-	return(match);
+	return match;
 }
 
-bool SyntaxHighlighter::isConfigurationLoaded(void)
+bool SyntaxHighlighter::isConfigurationLoaded()
 {
-	return(conf_loaded);
+	return conf_loaded;
 }
 
-void SyntaxHighlighter::clearConfiguration(void)
+void SyntaxHighlighter::clearConfiguration()
 {
 	initial_exprs.clear();
 	final_exprs.clear();
@@ -338,17 +383,14 @@ void SyntaxHighlighter::loadConfiguration(const QString &filename)
 		{
 			clearConfiguration();
 			xmlparser.restartParser();
-			xmlparser.setDTDFile(GlobalAttributes::TMPL_CONFIGURATIONS_DIR +
-								 GlobalAttributes::DIR_SEPARATOR +
-								 GlobalAttributes::OBJECT_DTD_DIR +
-								 GlobalAttributes::DIR_SEPARATOR +
-								 GlobalAttributes::CODE_HIGHLIGHT_CONF +
-								 GlobalAttributes::OBJECT_DTD_EXT,
-								 GlobalAttributes::CODE_HIGHLIGHT_CONF);
+			xmlparser.setDTDFile(GlobalAttributes::getTmplConfigurationFilePath(GlobalAttributes::ObjectDTDDir,
+																																					GlobalAttributes::CodeHighlightConf +
+																																					GlobalAttributes::ObjectDTDExt),
+													 GlobalAttributes::CodeHighlightConf);
 
 			xmlparser.loadXMLFile(filename);
 
-			if(xmlparser.accessElement(XMLParser::CHILD_ELEMENT))
+			if(xmlparser.accessElement(XmlParser::ChildElement))
 			{
 				do
 				{
@@ -356,27 +398,27 @@ void SyntaxHighlighter::loadConfiguration(const QString &filename)
 					{
 						elem=xmlparser.getElementName();
 
-						if(elem==ParsersAttributes::WORD_SEPARATORS)
+						if(elem==Attributes::WordSeparators)
 						{
 							xmlparser.getElementAttributes(attribs);
-							word_separators=attribs[ParsersAttributes::VALUE];
+							word_separators=attribs[Attributes::Value];
 						}
-						else if(elem==ParsersAttributes::WORD_DELIMITERS)
+						else if(elem==Attributes::WordDelimiters)
 						{
 							xmlparser.getElementAttributes(attribs);
-							word_delimiters=attribs[ParsersAttributes::VALUE];
+							word_delimiters=attribs[Attributes::Value];
 						}
-						else if(elem==ParsersAttributes::IGNORED_CHARS)
+						else if(elem==Attributes::IgnoredChars)
 						{
 							xmlparser.getElementAttributes(attribs);
-							ignored_chars=attribs[ParsersAttributes::VALUE];
+							ignored_chars=attribs[Attributes::Value];
 						}
-						else if(elem==ParsersAttributes::COMPLETION_TRIGGER)
+						else if(elem==Attributes::CompletionTrigger)
 						{
 							xmlparser.getElementAttributes(attribs);
 
-							if(attribs[ParsersAttributes::VALUE].size() >= 1)
-								completion_trigger=attribs[ParsersAttributes::VALUE].at(0);
+							if(attribs[Attributes::Value].size() >= 1)
+								completion_trigger=attribs[Attributes::Value].at(0);
 						}
 
 						/*	If the element is what defines the order of application of the groups
@@ -384,19 +426,19 @@ void SyntaxHighlighter::loadConfiguration(const QString &filename)
 								the groups used to highlight the source code. ALL groups
 								in this block must be declared before they are built
 								otherwise an error will be triggered. */
-						else if(elem==ParsersAttributes::HIGHLIGHT_ORDER)
+						else if(elem==Attributes::HighlightOrder)
 						{
 							//Marks a flag indication that groups are being declared
 							groups_decl=true;
 							xmlparser.savePosition();
-							xmlparser.accessElement(XMLParser::CHILD_ELEMENT);
+							xmlparser.accessElement(XmlParser::ChildElement);
 							elem=xmlparser.getElementName();
 						}
 
-						if(elem==ParsersAttributes::GROUP)
+						if(elem==Attributes::Group)
 						{
 							xmlparser.getElementAttributes(attribs);
-							group=attribs[ParsersAttributes::NAME];
+							group=attribs[Attributes::Name];
 
 							/* If the parser is on the group declaration block and not in the build block
 								 some validations are made. */
@@ -405,15 +447,15 @@ void SyntaxHighlighter::loadConfiguration(const QString &filename)
 								//Raises an error if the group was declared before
 								if(find(groups_order.begin(), groups_order.end(), group)!=groups_order.end())
 								{
-									throw Exception(Exception::getErrorMessage(ERR_REDECL_HL_GROUP).arg(group),
-													ERR_REDECL_HL_GROUP,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+									throw Exception(Exception::getErrorMessage(ErrorCode::InvRedeclarationGroup).arg(group),
+																	ErrorCode::InvRedeclarationGroup,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 								}
 								//Raises an error if the group is being declared and build at the declaration statment (not permitted)
-								else if(attribs.size() > 1 || xmlparser.hasElement(XMLParser::CHILD_ELEMENT))
+								else if(attribs.size() > 1 || xmlparser.hasElement(XmlParser::ChildElement))
 								{
-									throw Exception(Exception::getErrorMessage(ERR_DEF_INV_GROUP_DECL)
-													.arg(group).arg(ParsersAttributes::HIGHLIGHT_ORDER),
-													ERR_REDECL_HL_GROUP,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+									throw Exception(Exception::getErrorMessage(ErrorCode::InvGroupDeclaration)
+																	.arg(group).arg(Attributes::HighlightOrder),
+																	ErrorCode::InvRedeclarationGroup,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 								}
 
 								groups_order.push_back(group);
@@ -424,38 +466,38 @@ void SyntaxHighlighter::loadConfiguration(const QString &filename)
 								//Raises an error if the group is being constructed by a second time
 								if(initial_exprs.count(group)!=0)
 								{
-									throw Exception(Exception::getErrorMessage(ERR_DEF_DUPLIC_GROUP).arg(group),
-													ERR_DEF_DUPLIC_GROUP,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+									throw Exception(Exception::getErrorMessage(ErrorCode::DefDuplicatedGroup).arg(group),
+																	ErrorCode::DefDuplicatedGroup,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 								}
 								//Raises an error if the group is being constructed without being declared
 								else if(find(groups_order.begin(), groups_order.end(), group)==groups_order.end())
 								{
-									throw Exception(Exception::getErrorMessage(ERR_DEF_NOT_DECL_GROUP)
-													.arg(group).arg(ParsersAttributes::HIGHLIGHT_ORDER),
-													ERR_DEF_NOT_DECL_GROUP,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+									throw Exception(Exception::getErrorMessage(ErrorCode::DefNotDeclaredGroup)
+																	.arg(group).arg(Attributes::HighlightOrder),
+																	ErrorCode::DefNotDeclaredGroup,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 								}
 								//Raises an error if the group does not have children element
-								else if(!xmlparser.hasElement(XMLParser::CHILD_ELEMENT))
+								else if(!xmlparser.hasElement(XmlParser::ChildElement))
 								{
-									throw Exception(Exception::getErrorMessage(ERR_DEF_EMPTY_GROUP).arg(group),
-													ERR_DEF_EMPTY_GROUP,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+									throw Exception(Exception::getErrorMessage(ErrorCode::DefEmptyGroup).arg(group),
+																	ErrorCode::DefEmptyGroup,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 								}
 
-								chr_sensitive=(attribs[ParsersAttributes::CASE_SENSITIVE]==ParsersAttributes::_TRUE_);
-								italic=(attribs[ParsersAttributes::ITALIC]==ParsersAttributes::_TRUE_);
-								bold=(attribs[ParsersAttributes::BOLD]==ParsersAttributes::_TRUE_);
-								underline=(attribs[ParsersAttributes::UNDERLINE]==ParsersAttributes::_TRUE_);
-								partial_match=(attribs[ParsersAttributes::PARTIAL_MATCH]==ParsersAttributes::_TRUE_);
-								fg_color.setNamedColor(attribs[ParsersAttributes::FOREGROUND_COLOR]);
+								chr_sensitive=(attribs[Attributes::CaseSensitive]==Attributes::True);
+								italic=(attribs[Attributes::Italic]==Attributes::True);
+								bold=(attribs[Attributes::Bold]==Attributes::True);
+								underline=(attribs[Attributes::Underline]==Attributes::True);
+								partial_match=(attribs[Attributes::PartialMatch]==Attributes::True);
+								fg_color.setNamedColor(attribs[Attributes::ForegroundColor]);
 
 								//If the attribute isn't defined the bg color will be transparent
-								if(attribs[ParsersAttributes::BACKGROUND_COLOR].isEmpty())
+								if(attribs[Attributes::BackgroundColor].isEmpty())
 									bg_color.setRgb(0,0,0,0);
 								else
-									bg_color.setNamedColor(attribs[ParsersAttributes::BACKGROUND_COLOR]);
+									bg_color.setNamedColor(attribs[Attributes::BackgroundColor]);
 
-								if(!attribs[ParsersAttributes::LOOKAHEAD_CHAR].isEmpty())
-									lookahead_char[group]=attribs[ParsersAttributes::LOOKAHEAD_CHAR][0];
+								if(!attribs[Attributes::LookaheadChar].isEmpty())
+									lookahead_char[group]=attribs[Attributes::LookaheadChar][0];
 
 								format.setFontFamily(default_font.family());
 								format.setFontPointSize(default_font.pointSizeF());
@@ -473,7 +515,7 @@ void SyntaxHighlighter::loadConfiguration(const QString &filename)
 
 
 								xmlparser.savePosition();
-								xmlparser.accessElement(XMLParser::CHILD_ELEMENT);
+								xmlparser.accessElement(XmlParser::ChildElement);
 
 								if(chr_sensitive)
 									regexp.setCaseSensitivity(Qt::CaseSensitive);
@@ -487,25 +529,25 @@ void SyntaxHighlighter::loadConfiguration(const QString &filename)
 									if(xmlparser.getElementType()==XML_ELEMENT_NODE)
 									{
 										xmlparser.getElementAttributes(attribs);
-										expr_type=attribs[ParsersAttributes::TYPE];
-										regexp.setPattern(attribs[ParsersAttributes::VALUE]);
+										expr_type=attribs[Attributes::Type];
+										regexp.setPattern(attribs[Attributes::Value]);
 
-										if(attribs[ParsersAttributes::REGULAR_EXP]==ParsersAttributes::_TRUE_)
+										if(attribs[Attributes::RegularExp]==Attributes::True)
 											regexp.setPatternSyntax(QRegExp::RegExp2);
-										else if(attribs[ParsersAttributes::WILDCARD]==ParsersAttributes::_TRUE_)
+										else if(attribs[Attributes::Wildcard]==Attributes::True)
 											regexp.setPatternSyntax(QRegExp::Wildcard);
 										else
 											regexp.setPatternSyntax(QRegExp::FixedString);
 
 										if(expr_type.isEmpty() ||
-												expr_type==ParsersAttributes::SIMPLE_EXP ||
-												expr_type==ParsersAttributes::INITIAL_EXP)
+												expr_type==Attributes::SimpleExp ||
+												expr_type==Attributes::InitialExp)
 											initial_exprs[group].push_back(regexp);
 										else
 											final_exprs[group].push_back(regexp);
 									}
 								}
-								while(xmlparser.accessElement(XMLParser::NEXT_ELEMENT));
+								while(xmlparser.accessElement(XmlParser::NextElement));
 								xmlparser.restorePosition();
 							}
 						}
@@ -513,14 +555,14 @@ void SyntaxHighlighter::loadConfiguration(const QString &filename)
 
 					/* Check if there are some other groups to be declared, if not,
 							continues to reading to the other part of configuration */
-					if(groups_decl && !xmlparser.hasElement(XMLParser::NEXT_ELEMENT))
+					if(groups_decl && !xmlparser.hasElement(XmlParser::NextElement))
 					{
 						groups_decl=false;
 						xmlparser.restorePosition();
 					}
 
 				}
-				while(xmlparser.accessElement(XMLParser::NEXT_ELEMENT));
+				while(xmlparser.accessElement(XmlParser::NextElement));
 			}
 
 			itr=groups_order.begin();
@@ -534,8 +576,8 @@ void SyntaxHighlighter::loadConfiguration(const QString &filename)
 				if(initial_exprs[group].size()==0)
 				{
 					//Raises an error if the group was declared but not constructed
-					throw Exception(Exception::getErrorMessage(ERR_GROUP_DECL_NOT_DEFINED).arg(group),
-									ERR_GROUP_DECL_NOT_DEFINED,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+					throw Exception(Exception::getErrorMessage(ErrorCode::InvGroupDeclarationNotDefined).arg(group),
+													ErrorCode::InvGroupDeclarationNotDefined,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 				}
 			}
 
@@ -543,7 +585,7 @@ void SyntaxHighlighter::loadConfiguration(const QString &filename)
 		}
 		catch(Exception &e)
 		{
-			throw Exception(e.getErrorMessage(),e.getErrorType(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
+			throw Exception(e.getErrorMessage(),e.getErrorCode(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
 		}
 	}
 }
@@ -553,14 +595,14 @@ vector<QRegExp> SyntaxHighlighter::getExpressions(const QString &group_name, boo
 	map<QString, vector<QRegExp> > *expr_map=(!final_expr ? &initial_exprs : &final_exprs);
 
 	if(expr_map->count(group_name) > 0)
-		return(expr_map->at(group_name));
+		return expr_map->at(group_name);
 	else
-		return(vector<QRegExp>());
+		return vector<QRegExp>();
 }
 
-QChar SyntaxHighlighter::getCompletionTrigger(void)
+QChar SyntaxHighlighter::getCompletionTrigger()
 {
-	return(completion_trigger);
+	return completion_trigger;
 }
 
 void SyntaxHighlighter::setFormat(int start, int count, const QString &group)
