@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2018 - Raphael Araújo e Silva <raphael@pgmodeler.com.br>
+# Copyright 2006-2020 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,24 +25,24 @@ class RoleTest: public QObject {
     Q_OBJECT
 
   private slots:
-    void alterCommandEndsWithSemiColon(void);
+    void alterCommandEndsWithSemiColon();
 };
 
-void RoleTest::alterCommandEndsWithSemiColon(void)
+void RoleTest::alterCommandEndsWithSemiColon()
 {
   Role role1, role2;
   QString alter_cmd;
 
   role1.setName("role1");
-  role1.setOption(Role::OP_SUPERUSER, true);
+  role1.setOption(Role::OpSuperuser, true);
 
   role2.setName("role2");
-  role2.setOption(Role::OP_CREATEDB, false);
-  role2.setOption(Role::OP_CREATEROLE, false);
-  role2.setOption(Role::OP_LOGIN, false);
+  role2.setOption(Role::OpCreateDb, false);
+  role2.setOption(Role::OpCreateRole, false);
+  role2.setOption(Role::OpLogin, false);
 
   alter_cmd=role1.getAlterDefinition(&role2);
-  alter_cmd.remove(QString("\n%1\n").arg(ParsersAttributes::DDL_END_TOKEN));
+  alter_cmd.remove(QString("\n%1\n").arg(Attributes::DdlEndToken));
   QCOMPARE(alter_cmd.endsWith(";"), true);
 }
 
