@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2018 - Raphael Araújo e Silva <raphael@pgmodeler.com.br>
+# Copyright 2006-2020 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,10 +24,10 @@ class SyntaxHighlighterTest: public QObject {
     Q_OBJECT
 
   private slots:
-    void handleMultiLineComment(void);
+    void handleMultiLineComment();
 };
 
-void SyntaxHighlighterTest::handleMultiLineComment(void)
+void SyntaxHighlighterTest::handleMultiLineComment()
 {
   QDialog *dlg=new QDialog;
   QPlainTextEdit *edt=new QPlainTextEdit(dlg);
@@ -35,7 +35,7 @@ void SyntaxHighlighterTest::handleMultiLineComment(void)
   SyntaxHighlighter *sql_hl=nullptr;
 
   sql_hl=new SyntaxHighlighter(edt, false);
-  sql_hl->loadConfiguration(GlobalAttributes::SQL_HIGHLIGHT_CONF_PATH);
+	sql_hl->loadConfiguration(GlobalAttributes::getSQLHighlightConfPath());
 
   layout->addWidget(edt);
   dlg->exec();
